import createComponent from '../../../index.js';

const ENTER_KEY = 13;
const ESCAPE_KEY = 27;

export default createComponent({
  create(task) {
    this.model = task;
  },
  view() {
    const m = this.model;
    return (
      <li ref="li">
        <div class="view">
          <input class="toggle" type="checkbox" ref="toggle"/>
          <label ref="label">{m.label}</label>
          <button class="destroy" ref="remove"></button>
        </div>
      </li>
    );
  },
  viewEdit() {
    return <input class="edit" ref="edit" />;
  },
  render(container) {
    const { node, r } = this.view();
    this.refs = r;
    r.remove.addEventListener('click', this._onRemove);
    r.toggle.addEventListener('click', this._onComplete);
    r.toggle.checked = this.model.completed;
    r.li.classList.toggle('completed', this.model.completed);
    r.label.addEventListener('dblclick', this._dblClick);

    container.appendChild(node);
    return this;
  },
  detach() {
    const r = this.refs;
    r.li.parentElement.removeChild(r.li);
    r.remove.removeEventListener('click', this._onRemove);
    r.toggle.addEventListener('click', this._onComplete);
    this.model = null;
    this.refs = null;
    return this;
  },
  display(show) {
    this.refs.li.classList.toggle('hidden', !show);
  }
}, {
  // Events
  _onComplete(e) {
    const complete = e.target.checked;
    const li = this.refs.li;
    this.model.completed = complete;
    li.classList.toggle('completed', complete);
    this.trigger('updated', this);
  },
  _onRemove() {
    this.trigger('removed', this.model.id);
  },

  // Attach input
  _dblClick() {
    const { node, r } = this.viewEdit();
    this.refs.edit = r.edit;
    const ref = this.refs;

    ref.li.appendChild(node);
    ref.li.classList.toggle('editing', true);
    ref.edit.focus();
    ref.edit.value = this.model.label;
    ref.edit.addEventListener('blur', this._endEdit);
    ref.edit.addEventListener('keyup', this._checkKey);
  },
  _endEdit() {
    const r = this.refs;
    const e = r.edit;
    const nv = e.value;

    r.li.classList.toggle('editing', false);
    e.parentElement.removeChild(e);
    r.edit = null;

    if (this.model.label !== nv) {
      this.model.label = nv;
      r.label.textContent = nv;
      this.trigger('updated', this);
    }
  },
  _checkKey(e) {
    const r = this.refs;
    if (e.which === ENTER_KEY) {
      r.edit.blur();
    } else if (e.which === ESCAPE_KEY) {
      r.edit.value = r.label.textContent;
      r.edit.blur();
    }
  }
});
